###########################################################################################
##
## R source code to read and visualize Köppen-Geiger fields (Version of 19 August 2022)                                                                                    
##
## Rubel, F., K. Brugger, K. Haslinger, and I. Auer, 2017: The climate of the 
## European Alps: Shift of very high resolution Köppen-Geiger climate zones 1800-2100. 
## Meteorol. Z., DOI 10.1127/metz/2016/0816.
##
## (C) Climate Change & Infectious Diseases Group, University of Veterinary Medicine Vienna
##     Vetmeduni Vienna, Austria
##
###########################################################################################

# required packages 
library(raster); library(rasterVis); library(rworldxtra); data(countriesHigh)
library(latticeExtra)


# Read raster files
period='1986-2010'
r <- raster(paste('KG_', period, '.grd', sep=''))

# Color palette for climate classification
climate.colors=c("#960000", "#FF0000", "#FF6E6E", "#FFCCCC", "#CC8D14", "#CCAA54", "#FFCC00", "#FFFF64", "#007800", "#005000", "#003200", "#96FF00", "#00D700", "#00AA00", "#BEBE00", "#8C8C00", "#5A5A00", "#550055", "#820082", "#C800C8", "#FF6EFF", "#646464", "#8C8C8C", "#BEBEBE", "#E6E6E6", "#6E28B4", "#B464FA", "#C89BFA", "#C8C8FF", "#6496FF", "#64FFFF", "#F5FFFF")

# Legend must correspond to all climate classes, insert placeholders
r0 <- r[1:32]; r[1:32] <- seq(1,32,1)

# Converts raster field to categorical data
r <- ratify(r); rat <- levels(r)[[1]]

# Legend is always drawn in alphabetic order
 rat$climate <- c('Af', 'Am', 'As', 'Aw', 'BSh','BSk', 'BWh', 'BWk', 'Cfa', 'Cfb','Cfc', 'Csa', 'Csb', 'Csc', 'Cwa','Cwb', 'Cwc', 'Dfa', 'Dfb', 'Dfc','Dfd', 'Dsa', 'Dsb', 'Dsc', 'Dsd','Dwa', 'Dwb', 'Dwc', 'Dwd', 'EF','ET', 'Ocean')
 levels(r) <- rat

# Remove the placeholders
r[1:32] <- r0; # levels(r) <- rat

# Cities
lam=c(9.1859, 16.3745, 8.5348, 11.5797, 9.1818, 4.8317, 14.5046, 15.9812, 11.3401)
phi=c(45.4660, 48.2058, 47.3724, 48.1387, 48.7783, 45.7685, 46.0569, 45.8149, 44.4996)
city=c('Milan', 'Vienna', 'Zurich', 'Munich', 'Stuttgart', 'Lyon', 'Ljubljana', 'Zagreb', 'Bologna')
points=c('+', '+','+', '+','+','+', '+','+','+')

# Visualization (note the reduced resolution: maxpixels=ncell(r)/4)
if(.Platform$OS.type=="windows") {quartz<-function(...) windows(...)}
quartz(width=13, height=8.5, dpi=100)

print(levelplot(r, col.regions=climate.colors, xlab="", ylab="",
		scales=list(x=list(limits=c(xmin(r),xmax(r)), at=seq(xmin(r),xmax(r),1)), 
		y=list(limits=c(ymin(r), ymax(r)), at=seq(ymin(r), ymax(r), 1))), colorkey=list(space="top"), tck=0, maxpixels=ncell(r)/4)
		+latticeExtra::layer(sp.polygons(countriesHigh, lwd=1.0))
		+latticeExtra::layer(sp.text(cbind(lam, phi-0.2), city, col="#FFFFFF", cex=1.5))
		+latticeExtra::layer(sp.text(cbind(lam, phi), points, col="#FFFFFF", cex=1.5))
		+latticeExtra::layer(sp.text(cbind(5.6, 48.6), period, col="#FFFFFF", cex=2.0))
)

# Output of pdf			
dev.copy2pdf(file=paste('KG_', period,'.pdf', sep=''))

# Find the climate class for Vienna, Europe (or another location)
lam=  16.375; phi= 48.210
KG=r[cellFromXY(r, c(lam, phi))]
print(KG)